<?php

namespace Drupal\chat_messenger\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Chat Message Form for sending messages.
 */
class ChatMessageForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'chat_message_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $user = NULL) {
    // Get the current user.
    $current_user = \Drupal::currentUser();

    if ($current_user->id() == $user) {
      return [
        '#markup' => $this->t('You cannot chat with yourself.'),
      ];
    }

    $form['message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Type your message'),
      '#placeholder' => $this->t('Type your message here...'),
      '#required' => TRUE,
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Send'),
      '#ajax' => [
        'callback' => '::sendMessage',
        'wrapper' => 'chat-message-output',
      ],
      '#attributes' => [
        'data-recipient-id' => $user,
      ],
    ];

    $form['#prefix'] = '<div id="chat-message-output">';
    $form['#suffix'] = '</div>';
    $form['#cache']['max-age'] = 0;
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * Send message via AJAX.
   */
  public function sendMessage(array &$form, FormStateInterface $form_state) {
    $message = $form_state->getValue('message');
    $recipient_id = $form['actions']['submit']['#attributes']['data-recipient-id'];
    $sender_id = \Drupal::currentUser()->id();
    $created = \Drupal::time()->getRequestTime();

    \Drupal::database()->insert('chat_message')
      ->fields([
        'from_uid' => $sender_id,
        'to_uid' => $recipient_id,
        'message' => $message,
        'created' => $created,
      ])
      ->execute();

    $response = new AjaxResponse();

    $message_html = '<div class="message sent"><strong>' . $this->t('You:') . '</strong><p>' . $message . '</p></div>';
    $response->addCommand(new HtmlCommand('#chat-message-output', $message_html));

    $response->addCommand(new HtmlCommand('#edit-message', ''));
    $form['#cache']['max-age'] = 0;
    return $response;
  }

}
